#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl

use HTML::Template;
use ActiveState::Config qw( :path);
use PureMessage::Manager::Request::Safe ('PureMessage::Manager::Response::Quarantine::Enduser', import => [qw($REQUEST %in)] );
use PureMessage::Manager::Widget;
use PureMessage::Manager::Config qw(%config);

use PureMessage::Manager::Safe PureMessage::Enduser::Config, undef, qw( %enduser_config );
use PureMessage::Manager::Lang;
use PureMessage::Manager::Response::Quarantine::Enduser;
use PureMessage::List;
use strict;

my %vars;
our %config;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Quarantine::Enduser',
                                'stderr_to_stdout' =>1 );


my @METHODS = qw(email_session flat_file ldap);

my %PATHS = (
             'AUTH'=>['auth','#str','Authentication Method'],
             
             'AUTH_EMAIL_SESSION_TEMPLATE'=>['Authenticator#email_session.config.template','#str','Email template'],
             'AUTH_EMAIL_SESSION_EXPIRE'=>['Authenticator#email_session.config.session_expire','#duration','Session expiry time'],
             
             'AUTH_FLAT_FILE_PATH'=>['Authenticator#flat_file.config.file','#str','File path'],
             'AUTH_FLAT_FILE_CRYPT'=>['Authenticator#flat_file.config.crypt','#enum none crypt md5','Encryption<br>(can be none, crypt or md5)'],
             
             'AUTH_LDAP_SERVER'=>['Authenticator#ldap.config.ldap_server','#str','LDAP Server'],
             'AUTH_LDAP_BIND_DN'=>['Authenticator#ldap.config.bind_dn','#str','DN for binding to LDAP server'],
             'AUTH_LDAP_BIND_PASS'=>['Authenticator#ldap.config.bind_password','#str','Password for binding to LDAP server'],
             'AUTH_LDAP_MAIL_ATTR'=>['Authenticator#ldap.config.attribute_mail','#str','LDAP attribute for email address'],
             'AUTH_LDAP_BASE_DN'=>['Authenticator#ldap.config.base_dn','#str','Base DN for user accounts'],
             'AUTH_LDAP_FILTER'=>['Authenticator#ldap.config.filter','#str','Filter to find user account<br>(%%username%% is replaced with login)'],
           );

$r->{debug} = 0;
$r->add_help(text => "Help", tab => "quar", page => "mgr_quar_enduser_auth" );
$r->addRestartDlg(type=>'httpd');

$r->title(text('enduserf_auth_title'));

my $action = $in{'action'};

$action = 'save' if $in{'save'};
$action = 'edit' if $in{'edit'};
$action = 'test' if $in{'test'};

# get template vars we always want
$vars{'ACTION'} = $action;

# revisit is always set when the form is submitted so this is the first view
# so if load isn't sent from the link to this page we should still have
# valid values
#
$in{'load'} = 1 if !$in{revisit};


if ($action =~ /save/) {
    $vars{'TITLE'} = &text('enduserf_auth_title');
    $r->title(&text('enduserf_auth_title'));
    if ( !hasValidateErrors() ) {
         commitUpdatedDataToDisk();
         # allow PM::Service::Config to not exist, like
         # for instance, if they have an older PM-Utils
         # without it.
          eval {
                require PureMessage::Service::Config;
                PureMessage::Service::Config::stale('httpd');
              };

    } 
} 
elsif ($action =~ /test/) {
    if (!hasValidateErrors()) {
        testProvidedData();
    }
}

  putEditPage();


### start of subs


sub putEditPage() {

    $vars{HEADER} = $r->ThemeSectionHeader( title=> text('enduserf_auth_header'))->output(); 
    my $conf = PureMessage::Enduser::Config::edit();
    
    my @FEATURES;

    # Set defaults of "what we were given on the form"
    map { $vars{$_} = $in{$_} } keys %in;

    # Load values from cfg
    foreach my $fname (keys %PATHS) {
      my $value;
      if ($in{load}) {
        my $node = path_node($conf->root,$PATHS{$fname}->[0]);
        if ($fname eq 'AUTH_LDAP_SERVER') {
          # expecting an array
          $value = $node ? join(' ',$node->values()) : '';
        }
        else {
          $value = $node ? $node->value_formatted() : '';
        }
      }
      else {
          $value = $in{$fname};
      }
      if ($fname eq 'AUTH') {
          $vars{"AUTH_\U${value}"} = 'checked';
      }
      elsif ($value) {
          $vars{$fname} = $value;
      }
      $vars{"${fname}_LABEL"} = $PATHS{$fname}->[2];
    }
    for my $method (@METHODS) {
        my $node = path_node($conf->root, "Authenticator#$method.description");
        $vars{"\UAUTH_${method}_LABEL"} = $node ? $node->value_formatted() :
            "[${method}]";
    }
    my $tb = $r->ToolBar(
                        buttons => [[ { type=>'image',text=>'Save',name=>'save',icon=>'save'},
                                      { type=>'image',text=>'Cancel',href=>'../index.cgi',icon=>'cancel'},
                                      { type=>'image',text=>'Test',name=>'test',icon=>'continue'},
                                   ]],
                        table=> [[1,3],['50%','50%']],
                      );
    $vars{FOOTER} = $r->ThemeSectionFooter(toolbar=>$tb)->output();

    $r->add_template( 'file'=>'tmpl/auth.html', 'data'=>\%vars );
    $r->send();

}

sub commitUpdatedDataToDisk {
   
    my %values;
    
    eval {
      my $conf = PureMessage::Enduser::Config::edit();
      foreach my $k (keys %PATHS) {
        if ($k eq 'AUTH_LDAP_SERVER') {
            # put back into an array
            my @v = split(/\s+/, $in{$k});
            path_values($conf->root,$PATHS{$k}->[0],@v);
        }
        else {
            path_value($conf->root,$PATHS{$k}->[0],$in{$k});
        }
      }
      $conf->save();
      PureMessage::Enduser::Config::refresh();
    };

    if ($@) {
        # config died, deal with it
        $r->raise_error('error_index_saving_conf',$@);
        $r->error_dlg();
        $r->send();
        exit;
    }
    map { delete $in{$_} } grep /TEST/, keys %in; # Scrub user/pass before logging
    $r->addInfoBox( msg => text('message_update_ok'));
    $r->log('Update End User Auth','','enduser', \%in);
}

sub testProvidedData {
    my %values;
    eval {
      # Update a cfg based on provided values (that we're --NOT-- saving)
      my $conf = PureMessage::Enduser::Config::edit();
      foreach my $k (keys %PATHS) {
        if ($k eq 'AUTH_LDAP_SERVER') {
            # put back into an array
            my @v = split(/\s+/, $in{$k});
            path_values($conf->root,$PATHS{$k}->[0],@v);
        }
        else {
            path_value($conf->root,$PATHS{$k}->[0],$in{$k});
        }
      }
      # Initialize the authenticators with our test data
      my %auth_cfg = $conf->as_hash();
      require PureMessage::Enduser::Auth;
      PureMessage::Enduser::Auth::refresh( %auth_cfg );
      # Try to authenticate with provided user/pass
      my $user = $in{AUTH_TEST_USER};
      my $pass = $in{AUTH_TEST_PASS};
      my %info = PureMessage::Enduser::Auth::Authorize( $user, $pass );
      unless ($info{email}) {
        die text('enduserf_auth_test_fail') . "\n";
      }
      $r->addInfoBox( msg => text('enduserf_auth_test_ok') );
    };
    if ($@) {
      # raise error condition, but not as a modal dialog
      $r->raise_error('error_auth_test', $@);
    }
}


sub hasValidateErrors {
    foreach my $k (keys %PATHS) {
       $REQUEST->typedecl($k, $PATHS{$k}->[1],$PATHS{$k}->[2]);
    }
    my @v_errors = $REQUEST->validate(\%in);
    if (@v_errors) {
        $r->raise_error(join("<br>", @v_errors));
    } 
    return $r->error() ? 1 : 0;
}
