#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl

use HTML::Template;
use ActiveState::Config qw( :path);
use PureMessage::Manager::Request::Safe ('PureMessage::Manager::Response::Quarantine::Enduser', import => [qw($REQUEST %in)] );
use PureMessage::Manager::Config qw(%config);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Response::Quarantine::Enduser;

use PureMessage::Manager::Safe PureMessage::Enduser::Config, undef, qw( %enduser_config );
use PureMessage::Manager::Safe PureMessage::Enduser::Features, undef, qw( %enduser_config );
use PureMessage::Enduser::Config qw( %enduser_config );
use ActiveState::Config qw(:path);
use PureMessage::Lang qw(get_lang_map);
use PureMessage::Config qw(%config);
use strict;

my %features = ('quarantine.messages' => "View Quarantined Messages",
		'quarantine.body' => "View Message Contents",
		'list.white' => "User White List",
		'list.black' => "User Black List",
		'list.wildcard' => "White/Black List Wildcards",
		'option.spam_optout' => "Anti-Spam Opt-Out",
		'option.digest_optout' => "Digest Opt-Out",
		'option.vacation' => "Vacation Hold",
		'language.preference' => "Language Preference",
);

my @feature_order = qw(quarantine.messages quarantine.body list.white list.black list.wildcard option.spam_optout option.digest_optout option.vacation language.preference);

if ($enduser_config{auth} eq 'email_session') {
    $features{'email.notification'} = "Notification Email";
    push(@feature_order, "email.notification");
}   

my %vars;
our %config;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Quarantine::Enduser',
                                'stderr_to_stdout' =>1 );

$r->{debug} = 0;
$r->addRestartDlg(type=>'httpd');

$r->add_help(text => "Help", tab => "quar", page => "mgr_quar_enduser_feat" );

$r->title(text('enduserf_index_title'));

my $action = $in{'action'};

$action = 'save' if $in{'save'};
$action = 'edit' if $in{'edit'};

# get template vars we always want
$vars{'ACTION'} = $action;

# revisit is always set when the form is submitted so this is the first view
# so if load isn't sent from the link to this page we should still have
# valid values
#
$in{'load'} = 1 if !$in{revisit};

if ($action =~ /save/) {
    $r->title(&text('enduserf_index_title'));
    if ( !hasValidateErrors() ) {
         commitUpdatedDataToDisk();
         # allow PM::Service::Config to not exist, like
         # for instance, if they have an older PM-Utils
         # without it.
          eval {
                require PureMessage::Service::Config;
                PureMessage::Service::Config::stale('httpd');
              };

    } 
} 

putEditPage();


### start of subs


sub putEditPage() {

    $vars{HEADER} = $r->ThemeSectionHeader( width=>300, title=> text('enduserf_edit_header'))->output(); 
    
    my @FEATURES;
    my $real_fvalues = PureMessage::Enduser::Features::get_all();
    my $conf = PureMessage::Enduser::Config::edit();
    my $node = path_node($conf->root,'limit.vacation_expiry');
    $vars{MAX_DAYS} = $node ? $node->value_formatted() : 0;
    $vars{MAX_LIST} = $enduser_config{limit}->{max_list_items};
    foreach my $f (@feature_order) {
      push(@FEATURES,{ NAME => $f, FDESC => $features{$f}, VALUE=> $in{load} ? $real_fvalues->{$f} : $in{$f} });
    }
    $vars{FEATURES} = \@FEATURES;
    my @display_languages;
    my $curr_lang = $config{'language'};
    my %lang_map = get_lang_map();
    foreach my $key (keys(%lang_map)) {
	my $lang;
	if ((defined $curr_lang) && ($curr_lang eq $key)) {
	    $lang = {'selected' => 1, 'lang_abrev' => $key, 'lang_opt' => $lang_map{$key}->{'eng_lang'}};
	} else {
	    $lang = {'selected' => 0, 'lang_abrev' => $key, 'lang_opt' => $lang_map{$key}->{'eng_lang'}};
	}
	push (@display_languages, $lang);
    }
    $vars{'languages'} = \@display_languages;
    my $tb = $r->ToolBar(
                        buttons => [[ { type=>'image',text=>'Save',name=>'save',icon=>'save'},
                                      { type=>'image',text=>'Cancel',href=>'../index.cgi',icon=>'cancel'},
                                   ]],
                        table=> [[1,3],['50%','50%']],
                      );
    $vars{FOOTER} = $r->ThemeSectionFooter(toolbar=>$tb)->output();

    $r->add_template( 'file'=>'tmpl/index.html', 'data'=>\%vars );
    $r->send();

}

sub commitUpdatedDataToDisk {
   
    my %values;
    
    foreach my $f (keys %features) {
       $values{$f} = $in{$f} ? 1 : 0;
    }
    eval {
    # this is safe because we only allow features that appear in %features to bubble down
      PureMessage::Enduser::Features::set_features(%values); 

      my $conf = PureMessage::Enduser::Config::edit();
      path_value($conf->root,'limit.vacation_expiry',$in{max_days});
      path_value($conf->root,'limit.max_list_items',$in{max_list});
      $conf->save();
      PureMessage::Enduser::Config::refresh();
      $conf = PureMessage::Config->new('writable' => 1);
      path_value($conf->root, 'language', $in{def_lang});
      $conf->save();
    };

    if ($@) {
        # config died, deal with it
        $r->raise_error('error_index_saving_conf',$@);
        $r->error_dlg();
        $r->send();
        exit;
    }
    $r->addInfoBox( msg => text('message_update_ok'));
    $r->log('Update End User Features','','enduser', \%in);
}


sub hasValidateErrors {
    #$r->raise_error($) 
    #    unless $vars{max_days} =~ qr{^[0-9]+$};
    $in{max_days} = '' unless $in{max_days}; # validator won't check keys that don't exists in input
                                       # so force this to exist
    my %lang_map = get_lang_map();
    my $def_lang = "#value #enum " . join(' ', keys(%lang_map));
    $REQUEST->typedecl("def_lang", $def_lang, "Default Language");
    $REQUEST->typedecl("max_days", "#value #duration", "Max Hold Duration");
    $REQUEST->typedecl("max_list", "#value #int", "Max Items Per List");
    my @v_errors = $REQUEST->validate(\%in);
    if (@v_errors) {
        $r->raise_error(join("<br>", @v_errors));
    } 
    return $r->error() ? 1 : 0;
}
