package PureMessage::Enduser::Auth::Authenticator::FlatFile;
use PureMessage::Enduser::Auth::Authenticator;
use strict;
use PureMessage::Manager::Lang qw(mtext);
use Digest::MD5 qw(md5_base64);
our @ISA = qw(PureMessage::Enduser::Auth::Authenticator);

our %module_config;
our $PASSWORD_DB;
my $DEBUG = $ENV{AUTH_DEBUG};

sub init {
  %module_config = @_;
  $PASSWORD_DB = "$module_config{file}";
  $PASSWORD_DB = "$module_config{prefix}/etc/$PASSWORD_DB" unless $PASSWORD_DB =~ m(^/);
  die mtext('auth_flatfile_file_missing', $PASSWORD_DB,$!) unless -e $PASSWORD_DB and -r _;
  _init_file($PASSWORD_DB);
  return \&Authenticate;
}

sub _init_file{
 # this incase we decide to precache the file
}

# user,password,$email
sub _get_entry {
  my $username = shift;
  my $line = 0;
  open(FV,"$PASSWORD_DB") or die mtext('auth_flatfile_failed_to_open_file',$PASSWORD_DB,$!);
  while(<FV>) {
    if(/^\Q$username\E:/) {
      chomp;
      close FV;
      print "LineMatch($line): $_\n" if $DEBUG;
      my @lines = map { s/^\s+|\s+$//;$_} split(':',$_);
      return @lines; 
    }
    $line++;
  }
  close FV;
  return undef;
}

sub Authenticate {
  my ($in_user,$in_password,%opt) = @_;
  return unless $in_user and $in_password;
  print "Authenticate('$in_user','$in_password')\n" if $DEBUG;
  $opt{logger}->log('INFO',"authenticating $in_user with " .  __PACKAGE__ ) if $opt{logger};
  my ($f_user,$f_password,$f_email) = _get_entry($in_user);
  unless($f_user) {
    $opt{logger}->log('INFO',mtext('auth_flatfile_user_not_found',$in_user)) if $opt{logger};
    die mtext('auth_flatfile_user_not_found',$in_user) . "\n";
  }
  print "Found('$f_user','$f_password','$f_email')\n" if $DEBUG;
  $opt{logger}->log('INFO',"user '$f_user' is a valid user with an email of '$f_email'") if $opt{logger};
  if(_good_password($in_password,$f_password,%opt,user => $f_user)) {
    print "Authentication Success\n" if $DEBUG;
    $opt{logger}->log('INFO',"user '$f_user' authenticated successfully") if $opt{logger};
    $f_email = $in_user unless $f_email; # if there are only two fields assume the first is the email address
    return ( email => $f_email );
  } else {
    $opt{logger}->log('INFO',"user '$f_user' had the wrong password") if $opt{logger};
    print "Authentication Fail\n" if $DEBUG;
    return;
  }

}

sub _good_password {
  my ($in_password,$f_password,%opt) = @_;
  $opt{logger}->log('INFO',"user '$opt{user}' will be authenticated using a password of type '$module_config{crypt}'") if $opt{logger};
  if ($module_config{crypt} eq 'crypt') {
    print "Crypt(crypt)\n" if $DEBUG;
    return 1 if crypt($in_password,$f_password) eq $f_password;   
  } elsif ($module_config{crypt} eq 'md5') {
    print "Crypt(md5)\n" if $DEBUG;
    return 1 if md5_base64($in_password) eq $f_password;   
  } else {
    print "Crypt(none)\n" if $DEBUG;
    return 1 if $in_password eq $f_password;
  }
  return 0;
}


1;

