package PureMessage::Enduser::Auth::Email;
use PureMessage::Enduser::Session;
use PureMessage::Template qw 'expand_template';
use PureMessage::Enduser::Config qw(%config);
use PureMessage::Manager::Lang qw(mtext);
use PureMessage::Queue qw(queue_mail);
use PureMessage::Enduser::User;
use PureMessage::Config qw(expand_tmpl_filename);
use PureMessage::Resources::Manager;

# create a new Auth::Email session_id for an email and email it
#
=item GenerateKey($email_addr)
  Returns a new ID that can be used for Authorizing this user in the
  future.
=cut

sub _get_data_store {
    my $rm = PureMessage::Resources::Manager->new();
    my $ds = $rm->find('enduser_user_store');
    unless($ds) {
       $ds = $rm->add('enduser_user_store');
    }
    die "Unable to create resource 'enduser_user_store'\n" unless $ds;
    return $ds->native();
}


sub GenerateKey {
    my $email = shift;
    my $session = new PureMessage::Enduser::Session();
    return unless $email;
    $session->authorize(lc($email));
    $session->expire($config{Authenticator}->{email_session}->{config}->{session_expire})
      if $config{Authenticator}->{email_session}->{config}->{session_expire};
    my $ds = _get_data_store();
        my $user = $ds->get(lc($email));
    my $old_id = $user->retrieve('session_id');
    if ($old_id) {
      my $ts = PureMessage::Enduser::Session->restore($old_id);
      $ts->delete() if $ts;
    }
    $user->store('session_id',$session->id());
    return $session->id();
}
=item GetKey
  Attempts to retrieve this users session_id. Returns undef if this user
  doesn't have one
=cut

sub GetKey {
    my $email = shift;
    my $ds = _get_data_store();
    my $user = $ds->get(lc($email));
    my $old_id = $user->retrieve('session_id');
    return $old_id if $old_id;
    return;
}

sub EmailKey {
    my $email = shift;
    return unless $email;
    $email = lc($email);
    my $envfrom  = expand_template("%%ADMIN_ADDR:address%%");
    my %options = @_;
    my $session_id = GenerateKey($email);
    my $euconfig = $config{Authenticator}->{email_session}->{config};
    my %vars = ( session_id => $session_id,
                 mime_boundary => 'mime--1234567890--abcdefghijklmnopqrstuvwxyz--puremessage', 
                 enduser_url => $euconfig->{"destination"} || $config{'enduser_url'},
                 to => $email,
               );
    my $template = $options{template} || $euconfig->{template};
    # it's it's not absolute then it's relative to the etc dir 
    $template = expand_tmpl_filename($template, $PureMessage::Config::config{'language'}) unless $template =~ m{^/};
    open FV,"$template" or die mtext('auth_email_invalid_template',$template,$!);
    my $template_text;
    $template_text = join('',<FV>); close FV;
    my $email_text = expand_template($template_text,vars => \%vars);
    queue_mail(from => $envfrom,
               to => $email,
               message => $email_text
              );
              
    return $session_id;
}


1;

