package PureMessage::Enduser::Config;
use strict;
use PureMessage::Config qw(expand_filename);
use PureMessage::Resources::Manager;
use base qw(Exporter);
use PureMessage::Version qw($PREFIX);

our @EXPORT_OK = qw( %config $object %enduser_config );
our %config;

our $enduser_config; # alias that doesn't clash with other %configs
*enduser_config =\%config;

our $RESOURCE_MANAGER;

our $object;
our $real_PREFIX = $ENV{PREFIX} || $PREFIX; # for tests

sub refresh() {
    unless($RESOURCE_MANAGER) {
      $RESOURCE_MANAGER = PureMessage::Resources::Manager->new();

    }
    
    my $res = $RESOURCE_MANAGER->find('enduser_config');
    unless($res) {
        $res = $RESOURCE_MANAGER->add('enduser_config');
    }
    die "Unable to initialize enduser_config resource. Have the default resources been initialized?"
        unless $res;
    
    $object = $res->native();
    PureMessage::Enduser::Config::Validator->validate($object);
    $object->parse_file(); # the resource knows where the file is
    %config = $object->as_hash;
    foreach (qw( log_dir admin)) {
      $config{$_} = $PureMessage::Config::config{$_};
    }
    $config{prefix} = $PREFIX;
    \%config; # return this so the refresher can get it
}

refresh();

=item edit()

 Will open this config as a writeable  ActiveState::Config object
 and return it. This can die when configuration file has invalid
 syntax.

=cut

sub edit {
    
    unless($RESOURCE_MANAGER) {
      $RESOURCE_MANAGER = PureMessage::Resources::Manager->new();

    }
    my $res = $RESOURCE_MANAGER->find('enduser_config');
    unless($res) {
        $res = $RESOURCE_MANAGER->add('enduser_config');
    }
    die "Unable to initialize enduser_config resource. Have the default resources been initialized?"
        unless $res;
    
   my $p = $res->native( writable => 1);
 
   PureMessage::Enduser::Config::Validator->validate($p);
   $p->parse_file(); # the resource knows where the file is
   return $p;
}

1;


