package PureMessage::Enduser::Features;
use strict;
use warnings;
use PureMessage::Config;
use PureMessage::Enduser::Config qw(%enduser_config);
use ActiveState::Config qw(:path);


sub _get_path {
  my ($dest,$path,$hash) = @_;
  $path .= '.' if $path;
  foreach my $f (keys %$hash) {
     if (ref($hash->{$f}) eq 'HASH') {
       _get_path($dest,"$path$f",$hash->{$f});
     } else {
       $dest->{"$path$f"} = $hash->{$f};
     }
  }
}


sub get_all {
  my %features = ();
  _get_path(\%features,'',$PureMessage::Enduser::Config::config{feature});
  # canonize these values:
  foreach my $f (keys %features) {
    if ($features{$f} =~ /^true|yes|1$/) {
      $features{$f} = 1;
    } else {
      $features{$f} = 0; # this only allows boolean, should we allow other?
    }
  }
  
  $features{'email.notification'} = 0
      unless($enduser_config{auth} eq 'email_session');

  # If the optional multi authentication package is being used,
  # and one of the authentication methods is email_session, 
  # the email notification link needs to be displayed.
  if ($enduser_config{auth} eq 'multi') {
    my $auths = $enduser_config{Authenticator}{multi}{config}{authenticators};
    for my $a (@{$auths}) {        
      if ($a eq 'email_session') {
        $features{'email.notification'} = 1;
        last;
      } 
    }
  }
  
  return \%features;
}


sub set_features {
  my %features = @_;
  return unless keys %features;
  my $config = PureMessage::Enduser::Config::edit();
  my $root = $config->root();
  foreach my $f (keys %features) {
    # TODO maybe we should do a check to see if value are the same and skip if
    # they are?
    path_value($root,"feature.$f",$features{$f});
  }
  $config->save();
  PureMessage::Enduser::Config::refresh();
}




1;
