package PureMessage::Enduser::Request;
use strict;
use warnings;
use Exporter;
use CGI;
use ActiveState::Config qw (path_value);
use PureMessage::Manager::Lang qw(text);
use PureMessage::Manager::Response;
use PureMessage::Enduser::Response;
use CGI::Cookie;

our @ISA = qw(Exporter);

our %in;
our $CGI;
our %COOKIES;
our $in;
our @EXPORT_OK = qw(%COOKIES %in $in %text);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my ($FAKE_CGI,$FAKE_ENV) = (shift,shift);
    my %query;
    my %env;
    $CGI = new CGI; # global cgi
    %COOKIES = fetch CGI::Cookie;
    # get the lang namespace
    my $namespace = $CGI->url(-absolute => 1);
    $namespace =~ s/([^\/]+)$//;
    PureMessage::Manager::Lang::set_namespace($namespace);
   
    # get the CGI input values
    if (ref($FAKE_CGI) eq 'HASH') {

        # then it's the values we should be stuff into the object and pretend
        # came from the cgi
        %query = %$FAKE_CGI;
        %env = ref($FAKE_ENV) eq 'HASH' ? %$FAKE_ENV : ();

    } else {
        _parse_vars();
        # make a copy of global info for this object to use/abuse
        %query = %in;
        %env = %ENV;
    }

    my %self = (
                'query' => \%query,
                'env'   => \%env,
                'method' => _method(),
                'validator' => ActiveState::Config->new(),
                'cookies' => \%COOKIES,
                'cgi' => $CGI,
                @_
               );

    $self{validator}->typedecl("#nospaces", 
            sub { die "space characters are not allowed" if $_[3] =~ /\s/; }, undef );
    $self{validator}->typedecl("#nomarkup", 
            sub { die "markup symbols are not allowed" if $_[3] =~ /\<|\>|%/; }, undef );
    $self{validator}->typedecl("#value", 
            sub { die 'a non blank value is required' unless length($_[3]) }, undef );
    $self{validator}->typedecl("#word", 
            sub { die 'non alphanumeric characters are not allowed' if $_[3] =~ /\W/; }, undef );
    $self{validator}->typedecl("#as_option",
            sub { die 'must start with a letter or _ followed by zero or more alphanumeric characters and -'
                        if $_[3] !~ /^[a-z_][-\w]*$/i; },undef);
    $self{validator}->typedecl("#as_group_name",
            sub { die "must start with a letter or '_' followed by zero or more alphanumeric characters and '-' or ':'"
                        if $_[3] !~ /^[a-z_][-\w:]*$/i; },undef);
    return bless( \%self, $class);
}

sub typedecl {
    my $self = shift;
    my ($k, $v, $name) = @_;
    $self->{validator}->typedecl($k, $v, undef);
    if ( $name ) {
        $self->{labels} = {} unless $self->{labels};
        $self->{labels}{$k} = $name;
    }
}

sub validate {
    my $self = shift;
    my $query = shift || $self->{query};
    my @error;
    while ( my ($k, $v) = each(%$query) ) {
        eval {
            my $a = path_value($self->{validator}->root, $k, $v);
        };
        if ( $@ ) {
            my (undef, $err) = split(/:/, $@, 2);
            push @error, "Input error for the '$self->{labels}{$k}' field: " . ucfirst($err) . ".";
        }
    }
    return @error;
}

# constructs and returns a base response object
sub get_response {
    my $self = shift;
    if ( $self->{RESPONSE} ) { return $self->{RESPONSE} };
    my $class = shift || '';
    my @data = @_;
    my $r;
    push @data,  "REQUEST", $self;
    if ($class) {
        $r = $class->new(@data);
    }
    else {
        $r = PureMessage::Enduser::Response->new(@data);
    }
    $self->{RESPONSE} = $r;
    return $r;
}

sub query {
    my $self = shift;
    my $var = shift;
    return $self->{'query'}{$var};
}

sub query_names {
    my $self = shift;
    return keys %{$self->{'query'}};
}

sub _method {
    return $CGI->request_method();
}

sub _parse_vars {
    
    # Fills the %in hash with CGI parameters.
    my $meth = _method();
    %in = ();
    foreach my $k ($CGI->param()) {
        my @value = $CGI->param($k);
        my $value = $#value == 0 ? $value[0] : \@value; # we expect array refs when there are dups
        # strip off the .x and .y and set the key/value if not already
        # set. This is to make it easier to deal with image button
        # submits.
        if ($k =~ s/\.x|\.y$//) {
            $in{$k} = $value unless $in{$k};
        } else {
            $in{$k} = $value;
        }
    }

}

sub redirect
{
    my $self = shift;
    my ($port, $prot, $url);
    $port = $ENV{'SERVER_PORT'} == 443 && uc($ENV{'HTTPS'}) eq "ON" ? "" :
    $ENV{'SERVER_PORT'} == 80 && uc($ENV{'HTTPS'}) ne "ON" ? "" : ":$ENV{'SERVER_PORT'}";
    $prot = uc($ENV{'HTTPS'}) eq "ON" ? "https" : "http";
    if ($_[0] =~ /^(http|https|ftp|gopher):/) {
        $url = $_[0];
    }
    elsif ($_[0] =~ /^\//) {
        $url = "$prot://$ENV{'SERVER_NAME'}$port$_[0]";
    }
    elsif ($ENV{'SCRIPT_NAME'} =~ /^(.*)\/[^\/]*$/) {
        $url = "$prot://$ENV{'SERVER_NAME'}$port$1/$_[0]";
    }
    else {
        $url = "$prot://$ENV{'SERVER_NAME'}$port/$_[0]";
    }
    print "Location: $url\n\n";
}
                                                                                        


1;
