package PureMessage::Enduser::Session;
use PureMessage::Manager::Lang qw(mtext);
use PureMessage::Enduser::Config qw(%config);
use PureMessage::Resources::Manager;
use File::Path qw(mkpath);

use strict;

# create a new CGI::Session object with our settings
sub new {
  my $proto = shift;
  my $config = shift;
  my $self = _make_session_object($proto,undef,$config);
  return $self;
}

sub _make_session_object {
   my $proto = shift;
   my $session_id = shift;
   my $config = shift || \%config; # for testing
   my $args = $config->{session}->{args};
   my $rm = PureMessage::Resources::Manager->new();
   my $robj = $rm->find('eui_session');
   unless ($robj) {
      $robj = $rm->add('eui_session');
   }
   die "Unable to find a valid 'eui_session' resource\n" unless $robj;
   my $native = $robj->native();
   my $obj = $native->create_session($session_id);
   # because it is a #duration type it will be seconds, make it in the format
   # CGI::Session expects
   $obj->expire($args->{session_expire}) if defined $args->{session_expire};
   $obj->expire( $_ => $args->{expire}->{$_}) foreach keys %{$args->{expire}};
   
   my %self = ( _session => $obj );
   return bless(\%self,$proto);
   return $obj;
   
}

sub param {
   my $self = shift;
   return $self->{_session}->param(@_);
}

sub flush {
   my $self = shift;
   return $self->{_session}->flush(@_);
}

sub delete {
   my $self = shift;
   return $self->{_session}->delete(@_);
}

sub expire {
   my $self = shift;
   return $self->{_session}->expire(@_);
}

sub id {
   my $self = shift;
   return $self->{_session}->id(@_);
}

# attempt to restore a Session, return nothing if it isn't a valid one that
# we've marked up.
sub restore {
  my ($proto,$session_id,$session) = @_;
  my $self = _make_session_object($proto,$session_id,$session);
  unless($self and $self->param('_authorized')) {
      $self->delete() if $self;
      return;
  }
  return $self;
}

sub authorize{
  my $self = shift;
  my $email = shift;
  $self->param( email => $email);
  $self->param(_authorized => 1 );
  # make sure that the session is valid and writable, otherwise die
  if (!$self->flush()) {
      die $self->error();
  }
}

sub valid {
  my $self = shift;
  return 1 if $self->param("_authorized");
  return;
}

1;

