package PureMessage::RPC::Response::EnduserUI;
use PureMessage::RPC::Response::Enduser;
use PureMessage::Service;
use PureMessage::Enduser::Config qw( %config );
use PureMessage::Enduser::User;
use PureMessage::Enduser::Features;
use PureMessage::Manager::Config;
use PureMessage::RPC::Response;
use PureMessage::Quarantine;
use PureMessage::Manager::Lang qw(text mtext);
use PureMessage::Enduser::Session;
use PureMessage::RPC::NamedDataEncoder;
use Data::Dumper;

use strict;
use warnings;
our  @ISA = ('PureMessage::RPC::Response::Enduser');

# this object will collect and marshall all dispatches so that
# they appear as a single PureMessage::RPC::Data
# object

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = $class->SUPER::new(@_);
  return unless $self;
  $self->{'_data'} = PureMessage::RPC::NamedDataEncoder->new();
  return bless($self,$class);
}

sub dispatch {
  my ($self,@data) = @_;
  # figureout who called us
  my @stack = caller(1);
  my $caller = $stack[3] =~ /:{2}?([^:]+)$/ ? $1 : 'anon';
  $self->named_dispatch($caller,@data);
}

sub named_dispatch {
  my ($self,$name,@data) = @_;
  $self->SUPER::dispatch( $self->{'_data'}->create_dispatch_record($name,@data));
  return;
  @data = ( { UNDEF => 1 } ) unless scalar(@data); # dispatch an undef
  $self->SUPER::dispatch( { NAME=> $name, DATA => $self->{'_data'}->create_named_line($name,$_) . "\n" }) foreach @data;
}

sub raise_error {
  my ($self,$data) = @_;
  # figureout who called us
  my @stack = caller(1);
  my $caller = $stack[3] =~ /:{2}?([^:]+)$/ ? $1 : 'anon';
  $self->raise_detailed_error($caller,errorstr => $data);
}

sub raise_detailed_error {
  my ($self,$name,@extra) = @_;
  my $i=0;
  my $text = mtext($name,grep { $i++ % 2 } @extra);
  $self->log("dispatched error: " . $text);
  $self->SUPER::dispatch( $self->{'_data'}->create_error_record($name,{ errorstr=>$text,@extra }));
  $self->_dispatched_errors();
  return $text;
}

sub commit {
  my ($self) = @_;
  $self->SUPER::commit();
}

sub pg_messages {
  my ($self,$session_id,$view,$page_size,$page) = @_;
  $self->get_features($session_id);
  
  my $rows = $self->get_qview($session_id,$view);
  my @v = grep { $_->{view} eq $view} @$rows;
  return $self->named_dispatch('get_qrows',[]) unless scalar(@v) and $v[0]->{row_count};
  my $first = $page_size * $page <= $v[0]->{row_count} ? $page_size * $page : $v[0]->{row_count};
  my $last = $first + $page_size <= $v[0]->{row_count} ? $first + $page_size : $v[0]->{row_count};
  $self->get_qrows($session_id,$view,$first,$last);
  $self->named_dispatch('pg_messages',{ first => $first, last => $last, page => $page, page_size => $page_size });
}

# override some defaults with some extra data. 
# ** It is very important that calls that invalidate
# data that the UI has cached re-dispatches that data
# so the UI has a valid copy of it. For instance the
# UI is caching get_qviews, so if a method would cause
# that data to change it should redispatch a new 
# get_qview
#
sub get_qview {
  my ($self,@other) = @_;
  # get_qview is part of the default set to get
  $self->_always_get(@other);
  $self->_get_qview(@other);
}

sub init_qview {
  my ($self,@other) = @_;
  $self->_always_get(@other);
  $self->SUPER::init_qview(@other);
  $self->_get_qview(@other);
}

sub approve_msg {
  my ($self,@other) = @_;
  $self->_always_get(@other);
  $self->SUPER::approve_msg(@other);
  $self->_get_qview(@other);
}

sub delete_msg {
  my ($self,@other) = @_;
  $self->_always_get(@other);
  $self->SUPER::delete_msg(@other);
  $self->_get_qview(@other);
}

sub undelete_msg {
  my ($self,@other) = @_;
  $self->_always_get(@other);
  $self->SUPER::undelete_msg(@other);
  $self->_get_qview(@other);
}

sub getmsg_body {
  my ($self,@other) = @_;
  $self->_always_get(@other);
  $self->SUPER::getmsg_body(@other);
  $self->_get_qview(@other);
}

sub getmsg_meta {
  my ($self,@other) = @_;
  $self->_always_get(@other);
  $self->SUPER::getmsg_meta(@other);
  $self->_get_qview(@other);
}

sub get_items {
  my ($self,@other) = @_;
  $self->_always_get(@other);
  $self->_get_qview(@other);
  $self->SUPER::get_items(@other);
}

sub set_items {
  my ($self,$sid,$list,@other) = @_;
  $self->_always_get($sid);
  $self->SUPER::set_items($sid,$list,@other);
  $self->SUPER::get_items($sid,$list,@other);
  $self->_get_qview($sid);
}

sub get_attribute {
  my ($self,@other) = @_;
  $self->_always_get(@other);
  $self->SUPER::get_attribute(@other);
  $self->_get_qview(@other);
}

sub set_attribute {
  my ($self,$sid,%attributes) = @_;
  $self->_always_get($sid);
  $self->SUPER::set_attribute($sid,%attributes);
  $self->SUPER::get_attribute($sid); # get all attributes
  $self->_get_qview($sid);
}


sub _get_qview {
  my ($self,$sid,@other) = @_;
  return if $self->{_dispatched_get_qview};
  $self->SUPER::get_qview($sid);
  $self->SUPER::get_user($sid);
  $self->{_dispatched_get_qview} = 1;
}

sub _always_get {
  my ($self,$sid,@other) = @_;
  return if $self->{_dispatched_static};
  $self->SUPER::get_features($sid);
  $self->{_dispatched_always} = 1;
}

sub auth {
  my ($self,@other) = @_; 
 
  if (my $sid = $self->SUPER::auth(@other)) {
      $self->_always_get($sid);
      $self->_get_qview($sid);
  }
}


1;


